package org.eclipse.osgi.util;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.UnsupportedEncodingException;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.Hashtable;
import java.util.Map;
import java.util.StringTokenizer;
import org.eclipse.osgi.framework.internal.core.Tokenizer;
import org.osgi.framework.BundleException;

public class ManifestElement {
   private static final String MANIFEST_INVALID_SPACE = "The manifest line \"{0}\" has an invalid leading space '' '' character.";
   private static final String MANIFEST_INVALID_LINE_NOCOLON = "The manifest line \"{0}\" is invalid; it has no colon '':'' character after the header key.";
   private static final String MANIFEST_INVALID_HEADER_EXCEPTION = "Invalid manifest header {0}: \"{1}\"";
   protected String value;
   protected String[] valueComponents;
   protected Hashtable attributes;
   protected Hashtable directives;

   protected ManifestElement() {
   }

   public String getValue() {
      return this.value;
   }

   public String[] getValueComponents() {
      return this.valueComponents;
   }

   public String getAttribute(String key) {
      return this.getTableValue(this.attributes, key);
   }

   public String[] getAttributes(String key) {
      return this.getTableValues(this.attributes, key);
   }

   public Enumeration getKeys() {
      return this.getTableKeys(this.attributes);
   }

   protected void addAttribute(String key, String value) {
      this.attributes = this.addTableValue(this.attributes, key, value);
   }

   public String getDirective(String key) {
      return this.getTableValue(this.directives, key);
   }

   public String[] getDirectives(String key) {
      return this.getTableValues(this.directives, key);
   }

   public Enumeration getDirectiveKeys() {
      return this.getTableKeys(this.directives);
   }

   protected void addDirective(String key, String value) {
      this.directives = this.addTableValue(this.directives, key, value);
   }

   private String getTableValue(Hashtable table, String key) {
      if (table == null) {
         return null;
      } else {
         Object result = table.get(key);
         if (result == null) {
            return null;
         } else if (result instanceof String) {
            return (String)result;
         } else {
            ArrayList valueList = (ArrayList)result;
            return (String)valueList.get(valueList.size() - 1);
         }
      }
   }

   private String[] getTableValues(Hashtable table, String key) {
      if (table == null) {
         return null;
      } else {
         Object result = table.get(key);
         if (result == null) {
            return null;
         } else if (result instanceof String) {
            return new String[]{(String)result};
         } else {
            ArrayList valueList = (ArrayList)result;
            return (String[])valueList.toArray(new String[valueList.size()]);
         }
      }
   }

   private Enumeration getTableKeys(Hashtable table) {
      return table == null ? null : table.keys();
   }

   private Hashtable addTableValue(Hashtable table, String key, String value) {
      if (table == null) {
         table = new Hashtable(7);
      }

      Object curValue = table.get(key);
      if (curValue != null) {
         ArrayList newList;
         if (curValue instanceof ArrayList) {
            newList = (ArrayList)curValue;
         } else {
            newList = new ArrayList(5);
            newList.add(curValue);
         }

         newList.add(value);
         table.put(key, newList);
      } else {
         table.put(key, value);
      }

      return table;
   }

   public static ManifestElement[] parseHeader(String header, String value) throws BundleException {
      if (value == null) {
         return null;
      } else {
         ArrayList headerElements = new ArrayList(10);
         Tokenizer tokenizer = new Tokenizer(value);

         char c;
         label113:
         do {
            String next = tokenizer.getString(";,");
            if (next == null) {
               throw new BundleException(NLS.bind("Invalid manifest header {0}: \"{1}\"", header, value), 3);
            }

            ArrayList headerValues = new ArrayList();
            StringBuffer headerValue = new StringBuffer(next);
            headerValues.add(next);
            boolean directive = false;
            c = tokenizer.getChar();

            while(true) {
               do {
                  if (c != ';') {
                     ManifestElement manifestElement = new ManifestElement();
                     manifestElement.value = headerValue.toString();
                     manifestElement.valueComponents = (String[])headerValues.toArray(new String[headerValues.size()]);

                     while(c == '=' || c == ':') {
                        String val;
                        while(c == ':') {
                           c = tokenizer.getChar();
                           if (c != '=') {
                              val = tokenizer.getToken("=:");
                              if (val == null) {
                                 throw new BundleException(NLS.bind("Invalid manifest header {0}: \"{1}\"", header, value), 3);
                              }

                              next = next + ":" + c + val;
                              c = tokenizer.getChar();
                           } else {
                              directive = true;
                           }
                        }

                        val = tokenizer.getString(";,");
                        if (val == null) {
                           throw new BundleException(NLS.bind("Invalid manifest header {0}: \"{1}\"", header, value), 3);
                        }

                        try {
                           if (directive) {
                              manifestElement.addDirective(next, val);
                           } else {
                              manifestElement.addAttribute(next, val);
                           }

                           directive = false;
                        } catch (Exception var11) {
                           throw new BundleException(NLS.bind("Invalid manifest header {0}: \"{1}\"", header, value), 3);
                        }

                        c = tokenizer.getChar();
                        if (c == ';') {
                           next = tokenizer.getToken("=:");
                           if (next == null) {
                              throw new BundleException(NLS.bind("Invalid manifest header {0}: \"{1}\"", header, value), 3);
                           }

                           c = tokenizer.getChar();
                        }
                     }

                     headerElements.add(manifestElement);
                     continue label113;
                  }

                  next = tokenizer.getString(";,=:");
                  if (next == null) {
                     throw new BundleException(NLS.bind("Invalid manifest header {0}: \"{1}\"", header, value), 3);
                  }

                  c = tokenizer.getChar();

                  while(c == ':') {
                     c = tokenizer.getChar();
                     if (c != '=') {
                        String restOfNext = tokenizer.getToken(";,=:");
                        if (restOfNext == null) {
                           throw new BundleException(NLS.bind("Invalid manifest header {0}: \"{1}\"", header, value), 3);
                        }

                        next = next + ":" + c + restOfNext;
                        c = tokenizer.getChar();
                     } else {
                        directive = true;
                     }
                  }
               } while(c != ';' && c != ',' && c != 0);

               headerValues.add(next);
               headerValue.append(";").append(next);
            }
         } while(c == ',');

         if (c == 0) {
            int size = headerElements.size();
            if (size == 0) {
               return null;
            } else {
               ManifestElement[] result = (ManifestElement[])headerElements.toArray(new ManifestElement[size]);
               return result;
            }
         } else {
            throw new BundleException(NLS.bind("Invalid manifest header {0}: \"{1}\"", header, value), 3);
         }
      }
   }

   public static String[] getArrayFromList(String stringList) {
      String[] result = getArrayFromList(stringList, ",");
      return result.length == 0 ? null : result;
   }

   public static String[] getArrayFromList(String stringList, String separator) {
      if (stringList != null && stringList.trim().length() != 0) {
         ArrayList list = new ArrayList();
         StringTokenizer tokens = new StringTokenizer(stringList, separator);

         while(tokens.hasMoreTokens()) {
            String token = tokens.nextToken().trim();
            if (token.length() != 0) {
               list.add(token);
            }
         }

         return (String[])list.toArray(new String[list.size()]);
      } else {
         return new String[0];
      }
   }

   public static Map parseBundleManifest(InputStream manifest, Map headers) throws IOException, BundleException {
      if (headers == null) {
         headers = new HashMap();
      }

      BufferedReader br;
      try {
         br = new BufferedReader(new InputStreamReader(manifest, "UTF8"));
      } catch (UnsupportedEncodingException var14) {
         br = new BufferedReader(new InputStreamReader(manifest));
      }

      try {
         String header = null;
         StringBuffer value = new StringBuffer(256);
         boolean firstLine = true;

         while(true) {
            String line = br.readLine();
            if (line == null || line.length() == 0) {
               if (!firstLine) {
                  ((Map)headers).put(header, value.toString().trim());
               }
               break;
            }

            if (line.charAt(0) == ' ') {
               if (firstLine) {
                  throw new BundleException(NLS.bind("The manifest line \"{0}\" has an invalid leading space '' '' character.", (Object)line), 3);
               }

               value.append(line.substring(1));
            } else {
               if (!firstLine) {
                  ((Map)headers).put(header, value.toString().trim());
                  value.setLength(0);
               }

               int colon = line.indexOf(58);
               if (colon == -1) {
                  throw new BundleException(NLS.bind("The manifest line \"{0}\" is invalid; it has no colon '':'' character after the header key.", (Object)line), 3);
               }

               header = line.substring(0, colon).trim();
               value.append(line.substring(colon + 1));
               firstLine = false;
            }
         }
      } finally {
         try {
            br.close();
         } catch (IOException var13) {
         }

      }

      return (Map)headers;
   }

   public String toString() {
      Enumeration attrKeys = this.getKeys();
      Enumeration directiveKeys = this.getDirectiveKeys();
      if (attrKeys == null && directiveKeys == null) {
         return this.value;
      } else {
         StringBuffer result = new StringBuffer(this.value);
         String key;
         if (attrKeys != null) {
            while(attrKeys.hasMoreElements()) {
               key = (String)attrKeys.nextElement();
               this.addValues(false, key, this.getAttributes(key), result);
            }
         }

         if (directiveKeys != null) {
            while(directiveKeys.hasMoreElements()) {
               key = (String)directiveKeys.nextElement();
               this.addValues(true, key, this.getDirectives(key), result);
            }
         }

         return result.toString();
      }
   }

   private void addValues(boolean directive, String key, String[] values, StringBuffer result) {
      if (values != null) {
         for(int i = 0; i < values.length; ++i) {
            result.append(';').append(key);
            if (directive) {
               result.append(':');
            }

            result.append("=\"").append(values[i]).append('"');
         }

      }
   }
}
